@@ -1,19 +1,19 @@
-require 'puppet/application'
-require 'puppet/error'
-require 'puppet/util/at_fork'
+require_relative '../puppet/application'
+require_relative '../puppet/error'
+require_relative '../puppet/util/at_fork'
 
 require 'timeout'
 
 # A general class for triggering a run of another
 # class.
 class Puppet::Agent
-  require 'puppet/agent/locker'
+  require_relative 'agent/locker'
   include Puppet::Agent::Locker
 
-  require 'puppet/agent/disabler'
+  require_relative 'agent/disabler'
   include Puppet::Agent::Disabler
 
-  require 'puppet/util/splayer'
+  require_relative '../puppet/util/splayer'
   include Puppet::Util::Splayer
 
   # Special exception class used to signal an agent run has timed out.
@@ -64,10 +64,10 @@ class Puppet::Agent
             now = Time.now.to_i
             wait_for_lock_deadline ||= now + Puppet[:maxwaitforlock]
 
-            if Puppet[:waitforlock]  < 1
+            if Puppet[:waitforlock] < 1
               Puppet.notice _("Run of %{client_class} already in progress; skipping  (%{lockfile_path} exists)") % { client_class: client_class, lockfile_path: lockfile_path }
               nil
-            elsif now > wait_for_lock_deadline
+            elsif now >= wait_for_lock_deadline
               Puppet.notice _("Exiting now because the maxwaitforlock timeout has been exceeded.")
               nil
             else
